﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using Windows.ApplicationModel.DataTransfer;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Graphics.Printing;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.Storage.Streams;
using Windows.UI.Core;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;
using Windows.UI.Xaml.Printing;

// The Items Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234233

namespace Win8Contracts
{
    /// <summary>
    /// A page that displays a collection of item previews.  In the Split Application this page
    /// is used to display and select one of the available groups.
    /// </summary>
    public sealed partial class ItemsPage1 : Win8Contracts.Common.LayoutAwarePage
    {
		private PrintDocument _printDocument;
		private FileImage _selectedItem;
		private Image _image;

        public ItemsPage1()
        {
            this.InitializeComponent();

			// setup sharing source
			DataTransferManager dataTransferManager;
			dataTransferManager = DataTransferManager.GetForCurrentView();
			dataTransferManager.DataRequested += new TypedEventHandler<DataTransferManager, DataRequestedEventArgs>(this.DataRequested);

			PrintManager pm = PrintManager.GetForCurrentView();
			pm.PrintTaskRequested += pm_PrintTaskRequested;

        }

        private void pageRoot_Loaded(object sender, RoutedEventArgs e)
        {
			this.DefaultViewModel["Items"] = (App.Current as App).Images;
        }

		private async void Open_Click(object sender, RoutedEventArgs e)
		{
			FileOpenPicker picker = new FileOpenPicker();
			picker.ViewMode = PickerViewMode.Thumbnail;
			picker.SuggestedStartLocation = PickerLocationId.PicturesLibrary;
			picker.FileTypeFilter.Add(".png");
			picker.FileTypeFilter.Add(".jpg");
			StorageFile file = await picker.PickSingleFileAsync();
			if(file != null)
				await new MessageDialog("Selected: " + file.Name).ShowAsync();
			else
				await new MessageDialog("Cancelled!").ShowAsync();
		}

		private async void Save_Click(object sender, RoutedEventArgs e)
		{
			FileSavePicker picker = new FileSavePicker();
			picker.SuggestedStartLocation = PickerLocationId.PicturesLibrary;
			picker.FileTypeChoices.Add("PNG", new string[] {".png"});
			picker.FileTypeChoices.Add("JPG", new string[] { ".jpg" });
			StorageFile file = await picker.PickSaveFileAsync();
			if (file != null)
				await new MessageDialog("Selected: " + file.Name).ShowAsync();
			else
				await new MessageDialog("Cancelled!").ShowAsync();
		}

        private void DataRequested(DataTransferManager sender, DataRequestedEventArgs e)
		{
			// setup sharing data when requested
			FileImage fi = (FileImage)itemListView.SelectedItem;
			if(fi != null)
			{
				e.Request.Data.Properties.Title = "Sharing file";
				e.Request.Data.Properties.Description = fi.File.Path;
				e.Request.Data.SetStorageItems(new StorageFile[] { fi.File });
			}
		}

		private void pm_PrintTaskRequested(PrintManager sender, PrintTaskRequestedEventArgs args)
		{
			Dispatcher.Invoke(CoreDispatcherPriority.Normal, (x, y) =>
			{
				_selectedItem = (FileImage)itemListView.SelectedItem;

				_printDocument = new PrintDocument();
				_printDocument.AddPages += _printDocument_AddPages;
				_printDocument.GetPreviewPage += _printDocument_GetPreviewPage;

			}, this, null);

			if (_selectedItem != null)
			{
				PrintTask pt = args.Request.CreatePrintTask(_selectedItem.Title, (handlerArgs) =>
				{
					handlerArgs.SetSource(_printDocument.DocumentSource);
				});
				pt.Completed += pt_Completed;
				pt.Options.Orientation = PrintOrientation.Landscape;
			}
		}

		private void pt_Completed(PrintTask sender, PrintTaskCompletedEventArgs args)
		{
			Debug.WriteLine("completed");
		}

		private void _printDocument_GetPreviewPage(object sender, GetPreviewPageEventArgs e)
		{
			_image = new Image();
			_image.Source = _selectedItem.Image;
			_printDocument.SetPreviewPage(e.PageNumber, _image);
		}

		private void _printDocument_AddPages(object sender, AddPagesEventArgs e)
		{
			_printDocument.AddPage(_image);
			_printDocument.AddPagesComplete();
		}
    }
}
